#!/usr/bin/perl

#'********************************************************
#' Set warnings and diagnostics to output on browser
#'********************************************************
use warnings;
use diagnostics;
use CGI::Carp 'fatalsToBrowser';
#'********************************************************
#' Use filename basename function; utf8 character set and encoding
#'********************************************************
use File::Basename;
use utf8;
use encoding('utf8');
#'
#'********************************************************
#' My first web calculator
#'
#' Author     : Elario Belmontes, Jr.
#' Description: Four function calculator simulator for
#'              educational purposes to illustrate the 
#'              use of a web page on a personal web
#'              server
#' Languanges : Perl, JAVASCRIPT, JQUERY, HTML, CSS
#' File Name  : calc_42.pl
#' Disclaimer: This software is intended for educational learning
#'             purposes.
#'
#'             This software is provided "AS IS" and without any
#'             expressed or implied warranties, including, without
#'             limitation, the implied warranties of merchantability
#'             and fitness for a particular purpose.
#'
#'             No guarantee; No warranty; Install / Use at your own risk.
#'
#'
#' Copyright (C) 2015  Larry Belmontes, Jr. 
#'
#'
#'
#'********************************************************
#'
#'
#'********************************************************
#' Functions Section
#'********************************************************
#'
#'
#'********************************************************
#' Function: Do_Math
#' Math functions: add, subtract, multiply, divide
#'********************************************************/
sub Do_Math()
{
	if ($frm_NumIn gt ""){
		$frm_AccumTape = $frm_AccumTape . $frm_Session_NumberIn . $frm_Session_NumberInFunction;
		ComputeIT();
		$frm_AuditTape = $frm_AuditTape . $frm_NumIn . " " . $form_data{$frm_Control} . " " . chr(13) . chr(10);
		$frm_AccumTape = $frm_AccumTape . $frm_NumIn . "=" . $frm_Session_NumberIn . " " .  chr(13) . chr(10);
		$frm_Session_NumberInFunction = $form_data{$frm_Control};
		$frm_NumIn = ""
	}
	else
	{
		$ErrMsg = "No data entered...";
	}
	return;
}
#'
#'********************************************************
#' Function: Finalize_Math
#' Terminate Math function: equals key
#'********************************************************/
sub Finalize_Math()
{
	if ($frm_NumIn gt ""){
		$frm_AccumTape = $frm_AccumTape . $frm_Session_NumberIn . $frm_Session_NumberInFunction;
		ComputeIT();
		$frm_AuditTape = $frm_AuditTape . $frm_NumIn . " " . $form_data{$frm_Control} . " " . chr(13) . chr(10);
		$frm_AuditTape = $frm_AuditTape . $frm_Session_NumberIn . " T " . chr(13) . chr(10);
		$frm_AccumTape = $frm_AccumTape . $frm_NumIn . "=" . $frm_Session_NumberIn . " " . chr(13) . chr(10);
		$frm_AccumTape = $frm_AccumTape . "^^^^^^^^^^^^^^^^^^^^^^^ " . chr(13) . chr(10);
		$frm_NumIn = "";
		$frm_Session_NumberIn = "";
		$frm_Session_NumberInFunction = "";
	}
	else
	{
		$ErrMsg = "No data entered...";
	}
	return;
}
#'
#'********************************************************
#' Function: Clear_key
#' Housekeep: ClearAll key
#'********************************************************/
sub Clear_key()
{
	$frm_AuditTape = "";
	$frm_AccumTape = "";
	$frm_NumIn = "";
	$frm_Session_NumberIn = "";
	$frm_Session_NumberInFunction = "";
	return;
}
#'
#'********************************************************
#' Function: Power_Off_key
#' Housekeep: Power Off key
#'********************************************************/
sub Power_Off_key()
{
	#
	# No need to destroy session variables as they
	# exist on the HTML FORM
	#
	print "Content-type: text/html";
	print "\n\n";
	print "Powered Off!<br><br>Click ";
	print "<a title=" . chr(34) . "Power On Calculator" . chr(34);
	print " href=" . chr(34) . $myScriptName . chr(34) . "><font color=" . chr(34) . "#FFFFFF";
	print "><span style=" . chr(34) . "background-color: #0000CC" . chr(34);
	print ">here</span></font></a>" . " to power on myWeb Calculator...";
	return;
}
#'
#'********************************************************
#' Function: Power_On_key
#' Housekeep: Power On
#'********************************************************/
sub Power_On_key()
{
	$ErrMsg = "Power already ON...";
	return;
}
#'
#'********************************************************
#' Function: Clear_Entry_key
#' Housekeep: Clear entry 
#'********************************************************/
sub Clear_Entry_key()
{
	$frm_NumIn = "";
	return;
}
#'
#'********************************************************
#' Function: Numeric_key
#' Numeric entry
#'  Limit to n digits entry based on MaxInput excluding decimal point
#'********************************************************/
sub Numeric_key()
{
	if (($frm_NumIn =~ tr/.//) eq 0){ 
		#' no decimal found in number input
		#' check for MAXInput - 1
		if (length($frm_NumIn) > ($MaxInput - 1)){
			$ErrMsg = "Input limited to " . $MaxInput . " digits!";
		}
		else
		{
			$frm_NumIn = $frm_NumIn . $form_data{$frm_Control};
		}	
	}
	else
	{
		#' found decimal point in number input
		#'  check for MAXInput
		if (length($frm_NumIn) > $MaxInput){
			$ErrMsg = "Input limited to " . $MaxInput . " digits!";
		}
		else
		{
			$frm_NumIn = $frm_NumIn . $form_data{$frm_Control};
		}
	}
	return;
}
#'
#'********************************************************
#' Function: Decimal_key
#' Decimal point entry
#'********************************************************/
sub Decimal_key()
{
	if (($frm_NumIn =~ tr/.//) eq 0){
		$frm_NumIn = $frm_NumIn . ".";
	}
	return;
}
#'
#'********************************************************
#' Function: ComputeIT
#' Compute last result with current numeric entry
#'   based on math function
#'********************************************************/
sub ComputeIT()
{
	if ($frm_Session_NumberInFunction eq ""){
		$frm_Session_NumberIn = $frm_NumIn;
	}
	elsif ($frm_Session_NumberInFunction eq "+"){
		$frm_Session_NumberIn = $frm_Session_NumberIn + $frm_NumIn;
	}
	elsif ($frm_Session_NumberInFunction eq "-"){
		$frm_Session_NumberIn = $frm_Session_NumberIn - $frm_NumIn;
	}
	elsif ($frm_Session_NumberInFunction eq "x"){
		$frm_Session_NumberIn = $frm_Session_NumberIn * $frm_NumIn;
	}	
	elsif ($frm_Session_NumberInFunction eq "/"){
		$frm_Session_NumberIn = $frm_Session_NumberIn / $frm_NumIn;
	}
	return;
}
#'
#'********************************************************
#' Script Mainline 
#'********************************************************
#'
#'
#'********************************************************
#' Create/Resume a session
#'********************************************************
#'
# Session variables exist as INPUT hidden fields 
#' for this script
#'
#'********************************************************
#' Get my script file name
#'********************************************************
#'
$myScriptName = basename($ENV{'SCRIPT_NAME'});
#'
#'********************************************************
#' Initialize variables
#'********************************************************
#'
$myTitle = "myWeb Calculator";
$ErrMsg = "&nbsp;";
$MaxInput = 12;
$form = "";
#'
#'********************************************************
#' Process FORM post (i.e. calculator button pressed)
#'********************************************************
#'
if ($ENV{'REQUEST_METHOD'} eq "POST") {
	#'********************************************************
	#' Obtain data from FORM controls
	#'********************************************************
	read(STDIN, $form, $ENV{'CONTENT_LENGTH'});
	#'********************************************************
	#' Create associative array form_data
	#'********************************************************
	@name_value_pairs = split(/&/,$form);
	foreach $name_value (@name_value_pairs) {
		($name, $value) = split(/=/, $name_value);
		$value =~ tr/+/ /;
		$value =~ s/%(..)/pack("C", hex($1))/eg;
		$form_data{$name} = $value;
	}
	#'********************************************************
	#' Initialize variables
	#'********************************************************
	$frm_AuditTape = $form_data{'AuditTape'};
	$frm_NumIn = $form_data{'NumIn'};
	$frm_AccumTape = $form_data{'AccumTape'};
	#'********************************************************
	#' The following are used as session variables
	#'********************************************************
	$frm_Session_NumberIn = $form_data{'Session_NumberIn'};
	$frm_Session_NumberInFunction = $form_data{'Session_NumberInFunction'};
	#'
	#'********************************************************
	#' Button on FORM pressed -
	#'   Process number, function or housekeep button on form
	#'
	#'   Loop through each FORM field and take appropriate
	#'     action from POST data
	#'       (i.e.  AuditTape=&AccumTape=&NumIn=&B8=8)
	#'********************************************************/
	foreach $frm_Control (keys(%form_data)) {
		#'
		#'********************************************************
		#' Math functions: add, subtract, multiply, divide
		#'********************************************************/
		if ($frm_Control ~~ ["Fplus", "Fminus", "Fmult", "Fdiv"]){
				Do_Math();
		}
		#'
		#'********************************************************
		#' Terminate Math function: equals
		#'********************************************************/
		elsif ($frm_Control eq "Fequal"){
				Finalize_Math();
		}
		#'
		#'********************************************************
		#' Housekeep: Clear all
		#'********************************************************/
		elsif ($frm_Control eq "Fclear"){
				Clear_key();
		}
		#'
		#'********************************************************
		#' Housekeep: Power Off
		#'********************************************************
		elsif ($frm_Control eq "Poff"){
			Power_Off_key();
			exit();
		}
		#'
		#'********************************************************
		#' Housekeep: Power On
		#'********************************************************/
		elsif ($frm_Control eq "Pon"){
   			Power_On_key();
		}
   		#'
		#'********************************************************
		#' Housekeep: Clear entry
		#'********************************************************/
		elsif ($frm_Control eq "Fcleare"){
			Clear_Entry_key();
		}
		#'
		#'********************************************************
		#' Numeric entry
	   	#'  Limit to 16 digits entry excluding decimal point
		#'********************************************************/
   		elsif ($frm_Control ~~ ["B0", "B1", "B2", "B3", "B4", "B5", "B6", "B7", "B8", "B9"]){
			Numeric_key();
		}
		#'
		#'********************************************************
		#' Decimal point entry
		#'********************************************************/
   		elsif ($frm_Control eq "Bdot"){
			Decimal_key();
		}
	}
}	
else	
{
	#'
	#'********************************************************
	#' NOT a POST, initialize session variables 
	#'  Assume first time invoked
	#' ** Session variables are kept as hidden INPUT fields in form
	#'********************************************************/
	$frm_AuditTape = "";
	$frm_AccumTape = "";
	$frm_NumIn = "";
	$myTitle = "Welcome to myWeb Calculator and happy calculating!";
	$frm_Session_NumberIn = "";
	$frm_Session_NumberInFunction = "";	
}



#'
#'********************************************************
#' Browser output
#'********************************************************
#'
print "Content-type: text/html";
print "\n\n";

print <<HTMLStuff;

<html>

<head>

<style>
#t1 {color:green; font-size: 8pt; text-align: right;}
#t2 {color:red; font-size: 8pt; text-align: right;}
#btn {width:32; height:32;}
</style>

<title>myCalc</title>

<script src="https://ajax.googleapis.com/ajax/libs/jquery/2.1.3/jquery.min.js"></script>
<!--
<script type="text/javascript" src="Sys_web/Incl/jquery-latest.js">
</script>
-->
<!-- Script to scroll both result windows simultanously -->
<script type="text/javascript">
        \$(function() {
            \$('#t1').scroll(function() {
            \$('#t2')
                .scrollTop(\$('#t1').scrollTop());
            });
			\$('#t2').scroll(function() {
            \$('#t1')
                .scrollTop(\$('#t2').scrollTop());
            });
        });
    </script>
    
</head>

<body>

<!-- Calculator FORM -->
<form method="POST" action="$myScriptName" name="CalculateIT">
<!-- Place border on FORM and border title accordingly -->
<fieldset style="border-color:green; background-color:beige">
<legend style="color:blue; font-size: 18pt; background-color:lightsteelblue">$myTitle</legend>
<!-- Audit Trail and Intermediate Result textboxes -->
<table align="center">
<tr>
<td align="center">Audit Trail</td>
<td align="center">Intermediate Results</td>
</tr>
<tr>
<td><textarea id="t1" rows="15" name="AuditTape" cols="30"  readonly="readonly">$frm_AuditTape</textarea></td>
<td><textarea id="t2" rows="15" name="AccumTape" cols="50" readonly="readonly">$frm_AccumTape</textarea></td>
</tr>
</table>

<!-- Script to scroll both result windows to bottom of window simultaneously -->
<script type="text/javascript">
function Scroll_to_Bottom() 
{
document.getElementById('t1').scrollTop=document.getElementById('t1').scrollHeight;
document.getElementById('t2').scrollTop=document.getElementById('t2').scrollHeight;
}
// Call the function
Scroll_to_Bottom ();
    </script>

<!-- Audit Trail and Intermediate Result textboxes -->
<br/> 
<table border="1" cellpadding="10" cellspacing="0" align="center" style="border-collapse: collapse ; background-color:DarkSeaGreen">
<tr>
<td>
<!-- Place border on keyboard and display components -->
<fieldset style="border-color:green; background-color:lightsteelblue">
<!-- Calculator Display Title -->
$MaxInput Digit Display
<br/>
<!-- Calculator Display -->
	<input name="NumIn" size="15" value="$frm_NumIn" style="text-align:right; font-size: 10pt" readonly="readoly">
<!-- Calculator Error Message area -->
<br/>
<font color=red size="2">$ErrMsg</font>
<!-- Calculator Keypad -->
<br/>
      <input id="btn" type="submit" value="C"   name="Fclear" >&nbsp;
      <input id="btn" type="submit" value="CE"  name="Fcleare" >&nbsp;
      <input id="btn" type="submit" value="off" name="Poff" style="background-color: red; ">&nbsp;
      <input id="btn" type="submit" value="on"  name="Pon" style="background-color: green; color:lawngreen; ">&nbsp;
<br/>
      <input id="btn" type="submit" value="7"   name="B7" >&nbsp;
      <input id="btn" type="submit" value="8"   name="B8" >&nbsp;
      <input id="btn" type="submit" value="9"   name="B9" >&nbsp;
      <input id="btn" type="submit" value="/"   name="Fdiv" >&nbsp;
<br/>
      <input id="btn" type="submit" value="4"   name="B4" >&nbsp;
      <input id="btn" type="submit" value="5"   name="B5" >&nbsp;
      <input id="btn" type="submit" value="6"   name="B6" >&nbsp;
      <input id="btn" type="submit" value="x"   name="Fmult" >&nbsp;
<br/>
      <input id="btn" type="submit" value="1"   name="B1" >&nbsp;
      <input id="btn" type="submit" value="2"   name="B2" >&nbsp;
      <input id="btn" type="submit" value="3"   name="B3" >&nbsp;
      <input id="btn" type="submit" value="-"   name="Fminus" >&nbsp;
<br/>
      <input id="btn" type="submit" value="0"   name="B0" >&nbsp;
      <input id="btn" type="submit" value="."   name="Bdot" >&nbsp;
      <input id="btn" type="submit" value="="   name="Fequal" >&nbsp;
      <input id="btn" type="submit" value="+"   name="Fplus" >&nbsp;
</fieldset>
</td>
</tr>
</table>
</fieldset>
<input type="hidden" value="$frm_Session_NumberIn"   name="Session_NumberIn" >&nbsp;
<input type="hidden" value="$frm_Session_NumberInFunction"   name="Session_NumberInFunction" >&nbsp;
</form>
<u><i>myCalc</i></u> is for illustrative and educational purposes and does not represent a fully robust and tested calculator product.<br>
Copyright &copy; 2015  Larry Belmontes, Jr.
</body>

</html>

HTMLStuff

#print "<br/>";
#print $form . "len=" . $ENV{'CONTENT_LENGTH'};
#print "<br/>";
#foreach $item (keys(%form_data)) {
#	print "$item is set to $form_data{$item}<br>";
#}
#print "nn is $form_data{'nn'}<br>";
